--[[
	This is very much a homage to RangeHelp an addon that had this
	functionality and much more, but i realized that i didnt need
	anything other than the range button so i remade it in the
	image of ace2 with some extra spiffes.
    
    Original RangeHelp can be found at http://www.curse-gaming.com/en/wow/addons-1549-1-rangehelp.html
--]]

local Default = {
	width		= 135,
	height		= 35,
	textSize	= 12,
	pos	= {},
	locked = true
}

local Colors = {
	melee = { r=0, g=1, b=0, a=0.7 },
	range = { r=0, g=0, b=1, a=0.7 },
	deadzone = { r=1, g=0.5, b=0, a=0.7 },
	oorange = { r=1, g=0, b=0, a=0.7 }
}

Nudge = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")
local L = AceLibrary("AceLocale-2.2"):new("Nudge")

L:RegisterTranslations("enUS", function()
    return {
        ["Auto Shot"] = true,
        ["Wing Clip"] = true,
        ["Melee"] = true,
        ["Ranged"] = true,
        ["Dead Zone"] = true,
        ["Out of Range"] = true,
    }
end)

L:RegisterTranslations("koKR", function()
    return {
        ["Auto Shot"] = "자동 사격",
        ["Wing Clip"] = "날개 절단",
        ["Melee"] = "근접",
        ["Ranged"] = "원거리",
        ["Dead Zone"] = "위험 존",
        ["Out of Range"] = "거리 벗어남",
    }
end)

-- German translation comming from Guido Malpohl, much appreciated
L:RegisterTranslations("deDE", function()
    return {
        ["Auto Shot"] = "Automatischer Schuss",
        ["Wing Clip"] = "Zurechtstutzen",
        ["Melee"] = "Nahkampf",
        ["Ranged"] = "Fernkampf",
        ["Dead Zone"] = "Tote Zone",
        ["Out of Range"] = "Ausser Reichweite",
    }
end)

function Nudge:OnInitialize()

	local Options = {
		type = "group",
		args = {
	        lock = {
	            name = "lock",
	            type = "toggle",
	            desc = "Lock/Unlock the button.",
	            get = function() return self.db.profile.locked end,
	            set = function( v ) self.db.profile.locked = v end,
	            map = {[false] = "Unlocked", [true] = "Locked"},
	            guiNameIsMap = true,
            },
			width = {
				name = "Width", type = 'range', min = 10, max = 500, step = 1,
				desc = "Set the width of the button.",
				get = function() return self.db.profile.width end,
				set = function( v )
					self.db.profile.width = v
					self:Layout()
				end
			},
			height = {
				name = "Height", type = 'range', min = 5, max = 50, step = 1,
				desc = "Set the height of the button.",
				get = function() return self.db.profile.height end,
				set = function( v )
					self.db.profile.height = v
					self:Layout()
				end
			},
			font = {
				name = "Font", type = 'group',
				desc = "Set the font size of different elements.",
				args = {
					text = {
						name = "Text", type = 'range', min = 6, max = 32, step = 1,
						desc = "Set the font size on the button.",
						get = function() return self.db.profile.textSize end,
						set = function( v )
							self.db.profile.textSize = v
							self:Layout()
						end
					}
				}
			}
		}
	}
	
	self:RegisterDB("NudgeDB")
	self:RegisterDefaults('profile', Default)
	self:RegisterChatCommand( {"/nudge"}, Options )
	
end

function Nudge:OnEnable()

	self:CreateFrameWork()
	
	self:RegisterEvent("UNIT_FACTION", "TargetChanged")
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "TargetChanged")
		
end

function Nudge:SavePosition()

	local x, y = self.master:GetLeft(), self.master:GetTop()
	local s = self.master:GetEffectiveScale()
	
	self.db.profile.pos.x = x * s
	self.db.profile.pos.y = y * s

end

function Nudge:SetPosition()

	if self.db.profile.pos.x then
		local x = self.db.profile.pos.x
		local y = self.db.profile.pos.y

		local s = self.master:GetEffectiveScale()

		self.master:ClearAllPoints()
		self.master:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x / s, y / s)
	else
		self.master:ClearAllPoints()
		self.master:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	end

end

function Nudge:CreateFrameWork()	

	self.master = CreateFrame("Frame", "NudgeFrame", UIParent)
	self.master:Hide()
	
	self.master:SetMovable(true)
	self.master:EnableMouse(true)
	self.master:RegisterForDrag("LeftButton")
	self.master:SetScript("OnDragStart", function() if not self.db.profile.locked then self["master"]:StartMoving() end end)
	self.master:SetScript("OnDragStop", function() self["master"]:StopMovingOrSizing() self:SavePosition() end)
		
	self.master.Range = self.master:CreateFontString(nil, "OVERLAY")
    
    self:Layout()
	
end

function Nudge:Layout()

	self.master:SetWidth( self.db.profile.width )
	self.master:SetHeight( self.db.profile.height )

	self.master:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 8,
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", 
		edgeSize = 16,
		insets = {left = 4, right = 4, top = 4, bottom = 4},
	})
	
	self.master:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b)
	self.master:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
	
	local gameFont, _, _ = GameFontHighlightSmall:GetFont()
	self.master.Range:SetJustifyH("CENTER")
	self.master.Range:SetFont( gameFont, self.db.profile.textSize )
	self.master.Range:ClearAllPoints()
	self.master.Range:SetPoint("CENTER", self.master, "CENTER",0,0)
	self.master.Range:SetTextColor( 1,1,1 )
	
	self:SetPosition()
	
end

function Nudge:TargetChanged()

	if(UnitExists("target") and (not UnitIsDead("target")) and UnitCanAttack("player", "target")) then
		self.index = nil
		self:ScheduleRepeatingEvent("Nudge", self.OnUpdate, 0.1, self)
		self.master:Show()
	else
		self:CancelScheduledEvent("Nudge")
		self.master:Hide()
	end
	
end

function Nudge:OnUpdate()
	
	local text
	if IsSpellInRange(L["Wing Clip"]) == 1 then
		if Nudge.index ~= "melee" then
			text = L["Melee"]
			Nudge.index = "melee"
		else return end
	elseif IsSpellInRange(L["Auto Shot"]) == 1 then
		if Nudge.index ~= "range" then
			text = L["Ranged"]
			Nudge.index = "range"
		else return end
	elseif CheckInteractDistance("target", 4) then
		if Nudge.index ~= "deadzone" then
			text = L["Dead Zone"]
			Nudge.index = "deadzone";
		else return end
	else
		if Nudge.index ~= "oorange" then
			text = L["Out of Range"]
			Nudge.index = "oorange"
		else return end
	end

	Nudge.master:SetBackdropColor( 
		Colors[Nudge.index].r,
		Colors[Nudge.index].g,
		Colors[Nudge.index].b,
		Colors[Nudge.index].a
	)
		
	Nudge.master:SetBackdropBorderColor( 
		Colors[Nudge.index].r,
		Colors[Nudge.index].g,
		Colors[Nudge.index].b,
		Colors[Nudge.index].a
	)

	Nudge.master.Range:SetText( text )
	
end